#ifndef FUNCIONES_cpp
#define FUNCIONES_cpp

#include "funciones.h"


void inicializaTiraLed(CRGB tira_Leds[]){
  FastLED.addLeds<LED_CHIP, PIN_DATOS, ORDEN_COLOR_CHIP>(tira_Leds, NUM_LEDS_MAX);
}

void retardo (uint16_t milis){
  delay(milis);
}

void cambiarBrillo(uint8_t brillo){
  FastLED.setBrightness( brillo );
}


void rellenar(CRGB tira_Leds[], CRGB::HTMLColorCode color, uint8_t velocidad){
  for(int ledModificado=0; ledModificado<NUM_LEDS_MAX; ledModificado++){
    tira_Leds[ledModificado] = color;
    delay(velocidad);
    FastLED.show();
  }
}
void rellenarInversa(CRGB tira_Leds[], CRGB::HTMLColorCode color, uint8_t velocidad){
  for(int ledModificado=NUM_LEDS_MAX-1; ledModificado>=0; ledModificado--){
    tira_Leds[ledModificado] = color;
    delay(velocidad);
    FastLED.show();
  }
}


void estela(CRGB tira_Leds[], CRGB::HTMLColorCode color, uint8_t velocidad, uint8_t velocidadDegradado){
  for(int ledModificado=0; ledModificado<NUM_LEDS_MAX; ledModificado++){
    fadeToBlackBy( tira_Leds, NUM_LEDS_MAX, velocidadDegradado);
    tira_Leds[ledModificado] = color;
    delay(velocidad);
    FastLED.show();
  }
}

void estelaInversa(CRGB tira_Leds[], CRGB::HTMLColorCode color, uint8_t velocidad, uint8_t velocidadDegradado){
  for(int ledModificado=NUM_LEDS_MAX-1; ledModificado>=0; ledModificado--){
    fadeToBlackBy( tira_Leds, NUM_LEDS_MAX, velocidadDegradado);
    tira_Leds[ledModificado] = color;
    delay(velocidad);
    FastLED.show();
  }
}

void ping(CRGB tira_Leds[], CRGB::HTMLColorCode base, CRGB::HTMLColorCode alternativo, uint8_t velocidad){
  tira_Leds[0] = alternativo;
  delay(velocidad);
  FastLED.show();
  for(int ledModificado=1; ledModificado<NUM_LEDS_MAX; ledModificado++){
    tira_Leds[ledModificado-1] = base;
    tira_Leds[ledModificado] = alternativo;  
    delay(velocidad);
    FastLED.show();
  }
  tira_Leds[NUM_LEDS_MAX-1] = base;
  FastLED.show();
}

void pong(CRGB tira_Leds[], CRGB::HTMLColorCode base, CRGB::HTMLColorCode alternativo, uint8_t velocidad){
  tira_Leds[NUM_LEDS_MAX-1] = alternativo;
  delay(velocidad);
  FastLED.show();
  for(int ledModificado=NUM_LEDS_MAX-2; ledModificado>0; ledModificado--){
    tira_Leds[ledModificado+1] = base;
    tira_Leds[ledModificado] = alternativo;
    delay(velocidad);
    FastLED.show();
  }
  tira_Leds[0] = base;
  FastLED.show();
}


void rellenarDeUnoEnUno(CRGB tira_Leds[], CRGB::HTMLColorCode base, CRGB::HTMLColorCode alternativo, uint8_t velocidad){
  for(int limite=NUM_LEDS_MAX; limite>0; limite--){
    tira_Leds[0] = alternativo;
    delay(velocidad);
    FastLED.show();
    for(int ledModificado=1; ledModificado<limite; ledModificado++){
      tira_Leds[ledModificado-1] = base;      
      tira_Leds[ledModificado] = alternativo;
      delay(velocidad);
      FastLED.show();
    }
  }
  FastLED.show();
}
void rellenarDeUnoEnUnoInversa(CRGB tira_Leds[], CRGB::HTMLColorCode base, CRGB::HTMLColorCode alternativo, uint8_t velocidad){
  for(int limite=0; limite<NUM_LEDS_MAX; limite++){
    tira_Leds[NUM_LEDS_MAX-1] = alternativo;
    delay(velocidad);
    FastLED.show();
    for(int ledModificado=NUM_LEDS_MAX-2; ledModificado>=limite; ledModificado--){
      tira_Leds[ledModificado+1] = base;
      tira_Leds[ledModificado] = alternativo; 
      delay(velocidad);
      FastLED.show();
    }
  }
  FastLED.show();
}


void arcoirisFijo(CRGB tira_Leds[], uint8_t inicioEspectro, uint8_t longitudRepeticion) 
{
  fill_rainbow( tira_Leds, NUM_LEDS_MAX, inicioEspectro, longitudRepeticion);
  FastLED.show();
}


void arcoirisDesplazamiento(CRGB tira_Leds[],  uint8_t longitudRepeticion, uint8_t velocidad) 
{
  for(int inicioEspectro=0; inicioEspectro<256; inicioEspectro++){
    fill_rainbow( tira_Leds, NUM_LEDS_MAX, inicioEspectro, longitudRepeticion);
    delay(velocidad);
    FastLED.show();
  }
}


void fade(CRGB tira_Leds[], CRGB color, uint8_t velocidad){
  CRGB colorActual = tira_Leds[0];
  CRGB colorIntermedio=colorActual;
  while(color.red!=colorActual.red || color.green!=colorActual.green || color.blue!=colorActual.blue){
    colorActual = tira_Leds[0];
    if(colorActual.red>color.red){
      colorIntermedio.red=colorActual.red-1;
    }
    if(colorActual.red<color.red){
      colorIntermedio.red=colorActual.red+1;
    }

    if(colorActual.green>color.green){
      colorIntermedio.green=colorActual.green-1;
    }
    if(colorActual.green<color.green){
      colorIntermedio.green=colorActual.green+1;
    }

    if(colorActual.blue>color.blue){
      colorIntermedio.blue=colorActual.blue-1;
    }
    if(colorActual.blue<color.blue){
      colorIntermedio.blue=colorActual.blue+1;
    }
    for (int i=0;i<NUM_LEDS_MAX;i++){
      tira_Leds[ i ] = colorIntermedio;
    }
    FastLED.show();
    delay(velocidad);
  }
  
}

void test(CRGB tira_Leds[]){
  rellenar(tira_Leds, CRGB::Red,0);
  delay(3000);
  rellenar(tira_Leds, CRGB::Green,0);
  delay(3000);
  rellenar(tira_Leds, CRGB::Blue,0);
  delay(3000);
}


#endif
